"""
Testing the template tags
"""

# Django
from django.template import Context, Template
from django.test import TestCase

# Alliance Auth (External Libs)
from app_utils.urls import site_absolute_url

# AA Fleet Pings
from fleetpings import __version__


class TestVersionedStatic(TestCase):
    """
    Test fleetpings_versioned_static template tag
    """

    def test_versioned_static(self):
        """
        Test should return versioned static
        :return:
        """

        context = Context({"version": __version__})
        template_to_render = Template(
            "{% load fleetpings_versioned_static %}"
            "{% fleetpings_static 'fleetpings/css/fleetpings.min.css' %}"
        )

        rendered_template = template_to_render.render(context)

        self.assertInHTML(
            f'/static/fleetpings/css/fleetpings.min.css?v={context["version"]}',
            rendered_template,
        )


class TestReverseUrl(TestCase):
    """
    Test fleetpings_reverse_url template tag
    """

    def test_reverse_url(self):
        """
        Test should return a URL
        :return:
        """

        context = Context({"doctrine_pk": "1"})
        template_to_render = Template(
            "{% load fleetpings_urls %}"
            "{% fleetpings_reverse_url 'fittings:view_doctrine' doctrine_pk %}"
        )

        rendered_template = template_to_render.render(context)
        site_url = site_absolute_url()

        # self.assertEqual(rendered_template, 0)

        self.assertInHTML(
            f'{site_url}/fittings/doctrine/{context["doctrine_pk"]}/',
            rendered_template,
        )



"""
Tests for model FleetDoctrine
"""

# Django
from django.core.exceptions import ValidationError
from django.test import TestCase

# AA Fleet Pings
from fleetpings.models import FleetDoctrine


class TestModelFleetDoctrine(TestCase):
    """
    Testing the FleetDoctrine model
    """

    def test_should_return_fleetdoctrine_model_string_name(self):
        """
        Test should return the FleetDoctrine model string name
        :return:
        :rtype:
        """

        doctrine = FleetDoctrine(name="Awesome Ships")
        doctrine.save()

        self.assertEqual(str(doctrine), "Awesome Ships")

    def test_doctrine_link_should_throw_exception(self):
        """
        Test if we get a ValidationError for a doctrine link
        :return:
        """

        # given
        doctrine = FleetDoctrine(
            name="Awesome Doctrine", link="htp://invalid-doctrine.url"
        )

        # when
        with self.assertRaises(ValidationError):
            doctrine.clean()

        with self.assertRaisesMessage(
            ValidationError, expected_message="Your doctrine URL is not valid."
        ):
            doctrine.clean()



"""
Tests for model Setting
"""

# Django
from django.db.utils import IntegrityError
from django.test import TestCase

# AA Fleet Pings
from fleetpings.models import Setting
from fleetpings.tests.utils import create_setting


class TestSetting(TestCase):
    """
    Testing the Setting model
    """

    def test_model_string_name(self):
        """
        Test model string name
        :return:
        """

        # given
        setting = Setting.objects.get(pk=1)

        # when/then
        self.assertEqual(str(setting), "Fleet Pings Settings")

    def test_setting_save(self):
        """
        Test if there can't be another setting created
        and the existing setting is changed instead
        :return:
        """

        # given
        use_default_fleet_types = True
        use_default_ping_targets = True
        setting = Setting(
            pk=2,
            use_default_fleet_types=use_default_fleet_types,
            use_default_ping_targets=use_default_ping_targets,
        )
        setting.save()

        # then
        self.assertEqual(setting.pk, 1)
        self.assertEqual(setting.use_default_fleet_types, use_default_fleet_types)
        self.assertEqual(setting.use_default_ping_targets, use_default_ping_targets)

    def test_setting_create(self):
        """
        Test that create method throwing the following exception
        django.db.utils.IntegrityError: (1062, "Duplicate entry '1' for key 'PRIMARY'")
        :return:
        """

        # No pk given
        with self.assertRaises(IntegrityError):
            create_setting()

    def test_setting_create_with_pk(self):
        """
        Test that create method throwing the following exception no matter the given pk
        django.db.utils.IntegrityError: (1062, "Duplicate entry '1' for key 'PRIMARY'")
        :return:
        """

        # Set pk=2
        with self.assertRaises(IntegrityError):
            create_setting(pk=2)

    def test_cannot_be_deleted(self):
        """
        Test that the settings object cannot be deleted
        :return:
        """

        # given
        settings_old = Setting.objects.get(pk=1)

        # when
        Setting.objects.all().delete()

        # then
        settings = Setting.objects.all()
        settings_first = settings.first()

        # See if there is still only ONE Setting object
        self.assertEqual(settings.count(), 1)

        # Check if both of our objects are identical
        self.assertEqual(settings_old, settings_first)



"""
Hook into AA
"""

# Alliance Auth
from allianceauth import hooks
from allianceauth.services.hooks import MenuItemHook, UrlHook

# AA Fleet Pings
from fleetpings import __title__, urls


class AaFleetpingsMenuItem(MenuItemHook):  # pylint: disable=too-few-public-methods
    """
    This class ensures only authorized users will see the menu entry
    """

    def __init__(self):
        # Setup menu entry for sidebar
        MenuItemHook.__init__(
            self,
            __title__,
            "far fa-bell fa-fw",
            "fleetpings:index",
            navactive=["fleetpings:index"],
        )

    def render(self, request):
        """
        Check if the user has the permission to view this app
        :param request:
        :return:
        """

        if request.user.has_perm("fleetpings.basic_access"):
            return MenuItemHook.render(self, request)

        return ""


@hooks.register("menu_item_hook")
def register_menu():
    """
    Register our menu item
    :return:
    """

    return AaFleetpingsMenuItem()


@hooks.register("url_hook")
def register_urls():
    """
    Register our base url
    :return:
    """

    return UrlHook(urls, "fleetpings", r"^fleetpings/")
